library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALl;
use std.textio.ALL;
use work.i2c_tb_pkg.ALL;
use work.wishbone_driver_pkg.ALL;

entity image1_top_tb is
end image1_top_tb;

architecture behavior of image1_top_tb is

   --! ========================================================================
   --! Signals for the image1_top module
   --! ========================================================================
   signal wb_clk            : STD_LOGIC := '0';
   signal s_led_pw_o        : STD_LOGIC;
   signal s_led_err_o       : STD_LOGIC;
   signal s_led_ttl_o       : STD_LOGIC;
   signal s_pulse_led       : t_pulse_led_vector;
   signal s_pulse_i         : t_pulse_vector;
   signal s_pulse_o         : t_pulse_vector;
   signal s_inv_i           : STD_LOGIC_VECTOR(4 downto 1);
   signal s_inv_o           : STD_LOGIC_VECTOR(4 downto 1);
   --! Connections for i2c_slave_top
   signal s_sda_slave_oen   : STD_LOGIC;
   signal s_scl_slave_oen   : STD_LOGIC;
   signal s_I2C_master_i    : t_I2C_master_in;
   signal s_I2C_master_o    : t_I2C_master_out;
   signal s_I2C_slave_i     : t_I2C_slave_in;
   signal s_I2C_slave_o     : t_I2C_slave_out;
   signal s_FPGA_GA         : STD_LOGIC_VECTOR(4 downto 0);
   signal s_FPGA_GAP        : STD_LOGIC;
   --! Pins of the SPI interface to write into the Flash memory
   signal s_SPI_master_i    : t_SPI_master_in;
   signal s_SPI_master_o    : t_SPI_master_out;
   signal s_SPI_slave_i     : t_SPI_slave_in;
   signal s_SPI_slave_o     : t_SPI_slave_out;
   --! This LED will show the status of the PLL
   signal s_led_link_up_o   : STD_LOGIC;
   signal s_led_pps_o       : STD_LOGIC;
   signal s_led_wr_ok_o     : STD_LOGIC;
   signal s_fpga_en         : t_fpga_en;
   signal level             : STD_LOGIC;
   signal switch_i          : STD_LOGIC;
   signal manual_rst_n_o    : STD_LOGIC;
   signal s_RTM_id_i        : t_RTM_id;

   --! ========================================================================
   --! Signals for the i2c_master_driver (Renesasa alike)
   --! ========================================================================
   signal s_i2c_addr_op     : STD_LOGIC_VECTOR(7 downto 0)
                                        := (others => '0');
   signal s_wishbone_addr   : STD_LOGIC_VECTOR(c_WB_ADDR_LENGTH - 1 downto 0)
                                        := (others => '0');
   signal s_wr_data         : STD_LOGIC_VECTOR(c_WR_DATA_LENGTH - 1 downto 0)
                                        := (others => '0');
   signal s_rd_data         : STD_LOGIC_VECTOR(c_RD_DATA_LENGTH - 1 downto 0);
   signal s_i2c_driver_start          : STD_LOGIC := '0';
   signal s_i2c_driver_start_done     : STD_LOGIC;
   signal s_i2c_driver_pause          : STD_LOGIC := '0';
   signal s_i2c_driver_pause_done     : STD_LOGIC;
   signal s_i2c_driver_write          : STD_LOGIC := '0';
   signal s_i2c_driver_write_done     : STD_LOGIC;
   signal s_i2c_driver_read           : STD_LOGIC := '0';
   signal s_i2c_driver_read_done      : STD_LOGIC;


begin

   s_I2C_master_i  <=  s_I2C_slave_o;
   s_I2C_slave_i   <=  s_I2C_master_o;

   --! Clock process definitions
   wb_clk_process :process
   begin
      wb_clk <= '1';
        wait for c_WB_CLK_PERIOD/2;
      wb_clk <= '0';
        wait for c_WB_CLK_PERIOD/2;
   end process;

   -- Instantiate the Unit Under Test (UUT)

   i2c_driver: i2c_master_driver
   port map(tb_clk           => wb_clk,

            sda_master_i     => s_I2C_master_i.SDA,
            sda_master_o     => s_I2C_master_o.SDA,
            scl_master_o     => s_I2C_master_o.SCL,

            i2c_addr_op_i    => s_i2c_addr_op,
            wishbone_addr_i  => s_wishbone_addr,
            wr_data_i        => s_wr_data,
            rd_data_o        => s_rd_data,

            start_i          => s_i2c_driver_start,
            start_done_o     => s_i2c_driver_start_done,
            pause_i          => s_i2c_driver_pause,
            pause_done_o     => s_i2c_driver_pause_done,
            write_i          => s_i2c_driver_write,
            write_done_o     => s_i2c_driver_write_done,
            read_i           => s_i2c_driver_read,
            read_done_o      => s_i2c_driver_read_done);

   uut: image1_top
   generic map(g_NUMBER_OF_CHANNELS => c_NUMBER_OF_CHANNELS)
   port map(FPGA_CLK_P        =>,
            FPGA_CLK_N        =>,
            led_pw_o          => s_led_pw_o ,
            led_err_o         => s_led_err_o,
            led_ttl_o         => s_led_ttl_o,
            led_o_front       => s_pulse_led.FRONT,
            led_o_rear        => s_pulse_led.REAR,
            pulse_i_front     => s_pulse_i.FRONT,
            pulse_o_front     => s_pulse_o.FRONT,
            pulse_i_rear      => s_pulse_i.REAR,
            pulse_o_rear      => s_pulse_o.REAR,
            inv_i             => s_inv_i,
            inv_o             => s_inv_o,
            SCL_I             => s_I2C_master_o.SCL,
            SCL_O             => open,
            SCL_OE            => s_scl_slave_oen,
            SDA_I             => s_I2C_slave_i.SDA,
            SDA_O             => s_I2C_slave_o.SDA,
            SDA_OE            => s_sda_slave_oen,
            FPGA_GA           => s_FPGA_GA,
            FPGA_GAP          => s_FPGA_GAP,
            FPGA_PROM_CCLK    => s_SPI_master_o.CCLK,
            FPGA_PROM_CSO_B_N => s_SPI_master_o.CSO_B_N,
            FPGA_PROM_DIN     => s_SPI_master_i.DIN,
            FPGA_PROM_MOSI    => s_SPI_master_o.MOSI,
            led_link_up_o     => s_led_link_up_o,
            led_pps_o         => s_led_pps_o,
            led_wr_ok_o       => s_led_wr_ok_o,
            fpga_o_en         => s_fpga_en.GEN,
            fpga_o_blo_en     => s_fpga_en.BLO,
            fpga_o_ttl_en     => s_fpga_en.TTL,
            fpga_o_inv_en     => s_fpga_en.INV,
            level             => level,
            switch_i          => switch_i,
            manual_rst_n_o    => manual_rst_n_o,
            FPGA_RTMM         => s_RTM_id_i.RTMM,
            FPGA_RTMP         => s_RTM_id_i.RTMP);
   end component;



   --! Stimulus process
   p_i2c_tb: process

      procedure set_i2c_wb_feedback is
      begin
         s_feedback_wb_bus <= '1';
           wait until rising_edge(wb_clk);
      end procedure;

      procedure unset_i2c_wb_feedback is
      begin
         s_feedback_wb_bus <= '0';
           wait until rising_edge(wb_clk);
      end procedure;

      procedure start_I2C is
      begin
         s_i2c_driver_start <= '1';
           wait until rising_edge(s_i2c_driver_start_done);
           wait until rising_edge(wb_clk);
         s_i2c_driver_start <= '0';
      end procedure;

      procedure write_I2C(i2c_addr   : STD_LOGIC_VECTOR(7 downto 0);
            wb_addr : STD_LOGIC_VECTOR(s_wishbone_addr'length - 1 downto 0);
            wr_data : STD_LOGIC_VECTOR(s_wr_data'length       - 1 downto 0)) is
      begin
           wait until rising_edge(wb_clk);
         s_i2c_addr_op     <= i2c_addr;
         s_wishbone_addr   <= wb_addr;
         s_wr_data         <= wr_data;
         s_i2c_driver_write <= '1';
           wait until rising_edge(wb_clk);
         s_i2c_driver_write <= '0';
           wait until rising_edge(s_i2c_driver_write_done);
           wait until rising_edge(wb_clk);
      end procedure;

      --! This procedure is a wrapper in case we want to address with VME
      --! directions. Front padding is done with zeroes.
      procedure write_I2C(vme_slot   : UNSIGNED(4 downto 0);
            wb_addr : STD_LOGIC_VECTOR(s_wishbone_addr'length - 1 downto 0);
            wr_data : STD_LOGIC_VECTOR(s_wr_data'length       - 1 downto 0)) is
         variable v_i2c_addr  : STD_LOGIC_VECTOR(7 downto 0);
      begin
         v_i2c_addr(7 downto 6) := "00";
         v_i2c_addr(5 downto 1) := STD_LOGIC_VECTOR(vme_slot);
         v_i2c_addr(0)          := '0';
         write_I2C(v_i2c_addr, wb_addr, wr_data);
      end procedure;

      procedure read_I2C(vme_slot   : UNSIGNED(4 downto 0);
            wb_addr : STD_LOGIC_VECTOR(s_wishbone_addr'length - 1 downto 0);
            wr_data : STD_LOGIC_VECTOR(s_wr_data'length       - 1 downto 0)) is
         variable v_i2c_addr  : STD_LOGIC_VECTOR(7 downto 0);
      begin
         v_i2c_addr(7 downto 6) := "00";
         v_i2c_addr(5 downto 1) := STD_LOGIC_VECTOR(vme_slot);
         v_i2c_addr(0)          := '0';
           wait until rising_edge(wb_clk);
           -- i2c_master_driver
         s_i2c_addr_op     <= v_i2c_addr;
         s_wishbone_addr   <= wb_addr;
         s_wr_data         <= wr_data;
         s_i2c_driver_read <= '1';
           wait until rising_edge(wb_clk);
         s_i2c_driver_read <= '0';
           wait until rising_edge(s_i2c_driver_read_done);
           wait until rising_edge(wb_clk);
      end procedure;

   begin
      wb_rst_i <= '1';
        wait for c_WB_CLK_PERIOD*10;
      wb_rst_i <= '0';

      --! First we place a write
      set_i2c_wb_feedback;
      start_I2C;
      --! Try to write into DTX addr position
      write_I2C(c_VME_SLOT, X"0002", X"B16B00B5");
          wait for c_WB_CLK_PERIOD*25;

      --! Then we read from the FPGA
      --unset_i2c_wb_feedback;
      start_I2C;
      --! We read addr 1 (LT) via I2C
      --! Then, the core internally prefetchs via wishbone and then
      --! write the register into I2C to the master
      read_I2C(c_VME_SLOT, X"BAB1", X"B16B00B5");
        wait for c_WB_CLK_PERIOD*25;
   end process;

end;
